@@ -250,26 +250,28 @@ gboolean validVariableName( char *name )
 {
 	int len,num,i;
 	FUNCTION *Function  = cfgCalc->functions;
-	int	 nFunctions = cfgCalc->nFunctions;
+	
+	g_assert(name);
+	g_assert(cfgCalc->func_var_name);
 
 	len = strlen(name);
 	if( len<1 )
 	{
 		/* Engine | Bad variable name */
-		rc_error( _("variable name has zero length.") );
+		rc_error(RC_ERR_OTHER, _("variable name has zero length.") );
 		return FALSE;
 	}
 	
 	else if( len>MAX_VARIABLE_NAME_LEN )
 	{
 		/* Engine | Bad variable name */
-		rc_error( _("%s: variable name too long (>%d characters)."),
+		rc_error(RC_ERR_OTHER, _("%s: variable name too long (>%d characters)."),
 			name, MAX_VARIABLE_NAME_LEN );
 		return FALSE;
 	}
 	
 	else if( !strcasecmp( name, cfgCalc->func_var_name ) ) {
-	  rc_error( _("%s: variable name reserved for functions."),
+	  rc_error(RC_ERR_OTHER, _("%s: variable name reserved for functions."),
 		       name );
 		return FALSE;
 	}
@@ -277,7 +279,7 @@ gboolean validVariableName( char *name )
 	else if( !isalpha( name[0] ) )
 	{
 		/* Engine | Bad variable name */
-		rc_error( _("%s: variable name must start with a letter."),
+		rc_error(RC_ERR_OTHER, _("%s: variable name must start with a letter."),
 		       name );
 		return FALSE;
 	}
@@ -287,7 +289,7 @@ gboolean validVariableName( char *name )
 		if( is_correct(name[i]) ) continue;
 
 		/* Engine | Bad variable name */
-		rc_error( _("%s: variable name must be alphanumeric."), name );
+		rc_error(RC_ERR_OTHER, _("%s: variable name must be alphanumeric."), name );
 		return FALSE;
 	}
 
@@ -297,7 +299,7 @@ gboolean validVariableName( char *name )
 		if( strcasecmp( name, rCalc_engine_GetCommandName( i ) ) )
 			continue;
 		/* Engine | Bad variable name */
-		rc_error( _("%s: variable name is a reserved word."), name );
+		rc_error(RC_ERR_OTHER, _("%s: variable name is a reserved word."), name );
 		return FALSE;
 	}
 	for( i=0; i<NUM_DEFAULT_FUNCTIONS; i++ )
@@ -305,7 +307,7 @@ gboolean validVariableName( char *name )
 		if( strcasecmp( name, Function[i].name ) ) continue;
 
 		/* Engine | Bad variable name */
-		rc_error( _("%s: variable name is a reserved word."), name );
+		rc_error(RC_ERR_OTHER, _("%s: variable name is a reserved word."), name );
 		return FALSE;
 	}
 	for( i=0; i<NUM_CONSTANTS; i++ )
@@ -313,7 +315,7 @@ gboolean validVariableName( char *name )
 		if( strcasecmp( name, Constant[i].name ) ) continue;
 
 		/* Engine | Bad variable name */
-		rc_error( _("%s: variable name is a reserved word."), name );
+		rc_error(RC_ERR_OTHER, _("%s: variable name is a reserved word."), name );
 		return FALSE;
 	}
 
@@ -327,6 +329,7 @@ static gboolean setVariableValue( char *name, double value )
 
 	/* Check that its name is valid
 	*/
+	
 	if( !validVariableName(name) ) return FALSE;
 
 	/* If it already exists then overwrite it.
@@ -344,7 +347,7 @@ static gboolean setVariableValue( char *name, double value )
 			fparser_add_user_variable(name, value);*/
 
 			/*cfgCalc->variables_mod = TRUE;*/
-			
+			rc_export_replace_state(TRUE);
 			return TRUE;
 		}
 	}
@@ -355,7 +358,7 @@ static gboolean setVariableValue( char *name, double value )
 	if( cfgCalc->nVariables == MAX_VARIABLES )
 	{
 		/* Engine | too many variables! */
-		rc_error( _("out of variable storage space.") );
+		rc_error(RC_ERR_OTHER, _("out of variable storage space.") );
 		return FALSE;
 	}
 	
@@ -363,6 +366,7 @@ static gboolean setVariableValue( char *name, double value )
 	Variable[cfgCalc->nVariables].value = value;
 	/*fparser_add_user_variable(name, value);*/
 	cfgCalc->nVariables++;
+  rc_export_replace_state(FALSE);
 	
 	/*cfgCalc->variables_mod = TRUE;*/
 
@@ -390,7 +394,7 @@ static gboolean getVariableValue( char *name, double *value_p )
 		}
 	}
 	/* Engine | The specified variable does not exist */
-	rc_error( _("%s: undefined variable."), name );
+	rc_error(RC_ERR_UNDEFINED, _("%s: undefined variable."), name );
 	return FALSE;
 }
 
@@ -433,7 +437,7 @@ static gboolean pushString( char *str, Stack_t *stack )
 	if( stack->stack_i>=STACK_SIZE )
 	{
 		/* This should *never* occur */
-		rc_error( "out of stack space." );
+		rc_error(RC_ERR_OTHER, "out of stack space." );
 		return FALSE;
 	}
 	else
@@ -450,7 +454,7 @@ static gboolean pushNumber( double num, Stack_t *stack )
 	if( stack->stack_i>=STACK_SIZE )
 	{
 		/* This should *never* occur */
-		rc_error( "out of stack space." );
+		rc_error(RC_ERR_OTHER, "out of stack space." );
 		return FALSE;
 	}
 	else
@@ -467,7 +471,7 @@ static gboolean popString( char **str_p, Stack_t *stack )
 	if( stack->stack_i <= 0 )
 	{
 		/* Engine | The expression contains some kind of error */
-		rc_error( _("malformed expression.") );
+		rc_error(RC_ERR_OTHER, _("malformed expression.") );
 		return FALSE;
 	}
 	else
@@ -483,7 +487,7 @@ static gboolean popNumber( double *num_p, Stack_t *stack )
 	if( stack->stack_i <= 0 )
 	{
 	g_print("ici\n");
-		rc_error( _("malformed expression.") );
+		rc_error(RC_ERR_OTHER, _("malformed expression.") );
 		return FALSE;
 	}
 	else
@@ -500,7 +504,7 @@ static gboolean topString( char **str_p, Stack_t *stack )
   
 	if( stack->stack_i <= 0 )
 	{
-		rc_error( _("malformed expression.") );
+		rc_error(RC_ERR_OTHER, _("malformed expression.") );
 		return FALSE;
 	}
 	else
@@ -513,7 +517,7 @@ static gboolean topString( char **str_p, Stack_t *stack )
 /*  { */
 /*  	if( stack_i <= 0 ) */
 /*  	{ */
-/*  		rc_error( _("malformed expression.") ); */
+/*  		rc_error(RC_ERR_OTHER, _("malformed expression.") ); */
 /*  		return FALSE; */
 /*  	} */
 /*  	else */
@@ -575,7 +579,7 @@ static gboolean readItem( char **p, char **last, int *nInfixItems, char **exprIn
 	if( *nInfixItems==MAX_ITEMS )
 	{
 		/* This should *never* occur */
-		rc_error( "expresson contains too many terms (>%d).",
+		rc_error(RC_ERR_OTHER, "expresson contains too many terms (>%d).",
 			  MAX_ITEMS );
 		return FALSE;
 	}
@@ -635,7 +639,7 @@ static gboolean readItem( char **p, char **last, int *nInfixItems, char **exprIn
 				if(e)
 				{
 				/* Engine | A number is invalid in some way */
-					rc_error(_("%s: malformed number."),*p );
+					rc_error(RC_ERR_OTHER,_("%s: malformed number."),*p );
 					return FALSE;
 				}
 				else d=e=1;
@@ -643,13 +647,13 @@ static gboolean readItem( char **p, char **last, int *nInfixItems, char **exprIn
 				expr++;
 				if(*expr==0)
 				{
-					rc_error(_("%s: malformed number."),*p );
+					rc_error(RC_ERR_OTHER,_("%s: malformed number."),*p );
 					return FALSE;
 				}
 				if((*expr=='-')||(*expr=='+')) expr++;
 				if(!isdigit(*expr))
 				{
-					rc_error(_("%s: malformed number."),*p );
+					rc_error(RC_ERR_OTHER,_("%s: malformed number."),*p );
 					return FALSE;
 				}
 			}
@@ -658,14 +662,14 @@ static gboolean readItem( char **p, char **last, int *nInfixItems, char **exprIn
 				*expr = decimal;
 				if(d)
 				{
-					rc_error(_("%s: malformed number."),*p );
+					rc_error(RC_ERR_OTHER,_("%s: malformed number."),*p );
 					return FALSE;
 				}
 				else d=1;
 				expr++;
 				if(!isdigit(*expr))
 				{
-					rc_error(_("%s: malformed number."),*p );
+					rc_error(RC_ERR_OTHER,_("%s: malformed number."),*p );
 					return FALSE;
 				}
 			}
@@ -724,7 +728,7 @@ static gboolean readItem( char **p, char **last, int *nInfixItems, char **exprIn
 	}
 	else
 	{
-		rc_error( _("malformed expression.") );
+		rc_error(RC_ERR_OTHER, _("malformed expression.") );
 		return FALSE;
 	}
 
@@ -759,7 +763,7 @@ static gboolean breakInfix(char **expression, char **exprInfix)
 	*/
 	stripSpace( *expression );
 	if( !strlen(*expression) ) return FALSE;
-
+ 
 	/* Break the expression
 	*/
 	for( c=NULL, nInfixItems=0, expr=*expression;
@@ -789,7 +793,7 @@ static int priority_of( char operator )
 	case '(': return 6;
 	case ')': return 7;
 	default:
-		rc_error(_("internal error."));
+		rc_error(RC_ERR_OTHER,_("internal error."));
 		rc_debug( RC_DBG_ENGINE, "%c: illegal priority.", operator );
 		return 0;
 	}
@@ -805,7 +809,7 @@ static int priority_of( char operator )
 static gboolean infixToPostfix( Stack_t *stack, char **exprInfix, char **exprPostfix )
 {
 	char *Operators="F^/*+-()";
-	char *item,*p,*q;
+	char *item=NULL,*p,*q;
 	char **in,**out;
 
 	/* The stack should be empty - if not then we have a leak.
@@ -820,9 +824,10 @@ static gboolean infixToPostfix( Stack_t *stack, char **exprInfix, char **exprPos
 	*/
 	for( out=exprPostfix, in=exprInfix; *in; in++ )
 	{
-		item = *in;
+		g_assert(in);    
+    item = *in;
 
-		if( *item == '(' )
+    if( *item == '(' )
 		{
 			/* Push onto the stack
 			*/
@@ -868,7 +873,7 @@ static gboolean infixToPostfix( Stack_t *stack, char **exprInfix, char **exprPos
 		else
 		{
 			/* Engine | Programmer error */
-			rc_error( _("internal error.") );
+			rc_error(RC_ERR_OTHER, _("internal error.") );
 			rc_debug( RC_DBG_ENGINE, "%s: illegal item.", item );
 			return FALSE;
 		}
@@ -954,7 +959,7 @@ static gboolean evaluatePostfix( double *result_p, Stack_t *stack, char **exprPo
 				}
 				if( i==NUM_CONSTANTS )
 				{
-					rc_error( _("internal error.") );
+					rc_error(RC_ERR_OTHER, _("internal error.") );
 					rc_debug( RC_DBG_ENGINE,
 						  "%s: undefined constant.",
 						  (*postfix)+1 );
@@ -991,7 +996,7 @@ static gboolean evaluatePostfix( double *result_p, Stack_t *stack, char **exprPo
 					  if( !pushNumber(res))
 					        return FALSE;
 				          }
-					  else {rc_error( _("internal error.")); return FALSE;} */
+					  else {rc_error(RC_ERR_OTHER, _("internal error.")); return FALSE;} */
 					  res = evaluateFunction(Function[i].expression, x);
 					  if( !pushNumber(res, stack))
 					        return FALSE;
@@ -1003,7 +1008,7 @@ static gboolean evaluatePostfix( double *result_p, Stack_t *stack, char **exprPo
 				}
 				if( i==nFunctions )
 				{
-					rc_error( _("internal error.") );
+					rc_error(RC_ERR_OTHER, _("internal error.") );
 					rc_debug( RC_DBG_ENGINE,
 						  "%s: undefined function.",
 					       (*postfix)+1 );
@@ -1043,7 +1048,7 @@ static gboolean evaluatePostfix( double *result_p, Stack_t *stack, char **exprPo
 				if( y==0.0 )
 				{
 					/* Engine | division by zero */
-					rc_error( _("division by zero.") );
+					rc_error(RC_ERR_OTHER, _("division by zero.") );
 					return FALSE;
 				}
 				if( !pushNumber(x/y, stack) )	return FALSE;
@@ -1056,7 +1061,7 @@ static gboolean evaluatePostfix( double *result_p, Stack_t *stack, char **exprPo
 				break;
 
 			default:
-				rc_error( _("internal error.") );
+				rc_error(RC_ERR_OTHER, _("internal error.") );
 				rc_debug( RC_DBG_ENGINE,
 					  "%d: illegal item.", *postfix[0] );
 				return FALSE;
@@ -1068,7 +1073,7 @@ static gboolean evaluatePostfix( double *result_p, Stack_t *stack, char **exprPo
 	*/
 	if( stackSize(stack)!=1 )
 	{
-		rc_error( _("malformed expression.") );
+		rc_error(RC_ERR_OTHER, _("malformed expression.") );
 		rc_debug( RC_DBG_ENGINE,
 			  "%d items on stack after evaluation.", stackSize(stack) );
 		return FALSE;
@@ -1098,7 +1103,7 @@ gdouble evaluateFunction(gchar *formula, gdouble where)
 	if( strlen(formula)>MAX_EXPRESSION_LEN )
 	{
 		/* Engine | the expression is too long */
-		rc_error( _("expression too long (>%d characters)"),
+		rc_error(RC_ERR_OTHER, _("expression too long (>%d characters)"),
 		       MAX_EXPRESSION_LEN );
 		return;
 	}
@@ -1120,7 +1125,7 @@ gdouble evaluateFunction(gchar *formula, gdouble where)
 		if( strchr( rhs,'=' ) )
 		{
 			/* Engine | too many '=' in expression */
-			rc_error( _("too many equalities.") );
+			rc_error(RC_ERR_OTHER, _("too many equalities.") );
 			return;
 		}
 
@@ -1138,20 +1143,44 @@ gdouble evaluateFunction(gchar *formula, gdouble where)
 	return result;
 }
 
-/* will be introduced in next version ! ;-) */
 #if 0
+gboolean rc_verifyFunction(gchar *formula, RcEngine *localEngine)
+{
+  char *expr = g_strdup(formula);
+  char *pc;
+  char TESTVALUE='1';
+  RcLastRes RC_result;
+    
+  pc = expr;
+    
+  while (pc != '\0') 
+  {
+    if (*pc == 'x') *pc=TESTVALUE;
+    pc+=1;
+  }
+  rc_engine_execute(localEngine, expr);
+  RC_result = rc_import_lastres();
+  
+  if ( RC_result.expType == RC_EXP_ERROR)
+    return FALSE;
+
+	else return TRUE;
+}
+#endif
+
 gboolean verifyFunction(gchar *formula)
 {
-  gboolean result;
+  double result;
   Stack_t stack;
-  char *lhs,*rhs;
+  char *rhs;
   char *name;
   char  exprBuffer [MAX_EXPRESSION_LEN*2];
   char *expression;
   char *exprInfix  [MAX_ITEMS+1];
   char *exprPostfix[MAX_ITEMS+1];
+  gdouble TESTVALUE=1;
   
- 	setUnknownValue(0);
+ 	setUnknownValue(TESTVALUE);
   
 	stack.stack_i=0;
 	
@@ -1160,7 +1189,7 @@ gboolean verifyFunction(gchar *formula)
 	if( strlen(formula)>MAX_EXPRESSION_LEN )
 	{
 		/* Engine | the expression is too long */
-		rc_error( _("expression too long (>%d characters)"),
+		rc_error(RC_ERR_OTHER, _("expression too long (>%d characters)"),
 		       MAX_EXPRESSION_LEN );
 		return;
 	}
@@ -1180,11 +1209,11 @@ gboolean verifyFunction(gchar *formula)
 	*/
 	if( !breakInfix(&expression, exprInfix) ) return FALSE;
 	if( !infixToPostfix(&stack, exprInfix, exprPostfix) )  return FALSE;
-	/*if( !evaluatePostfix( &result, &stack, exprPostfix ) ) return FALSE;*/
+	if( !evaluatePostfix( &result, &stack, exprPostfix ) ) return FALSE;
 
 	return TRUE;
 }
-#endif
+
 
 void rCalc_engine_Evaluate( char *input )
 {
@@ -1194,17 +1223,18 @@ void rCalc_engine_Evaluate( char *input )
 	Stack_t stack;
 	char  exprBuffer [MAX_EXPRESSION_LEN*2];
 	char *expression;
+	gchar *init_expression;
 	char *exprInfix  [MAX_ITEMS+1];
 	char *exprPostfix[MAX_ITEMS+1];
 
 	stack.stack_i=0;
-	
+
 	/* Make a copy of the input string
 	*/
 	if( strlen(input)>MAX_EXPRESSION_LEN )
 	{
 		/* Engine | the expression is too long */
-		rc_error( _("expression too long (>%d characters)"),
+		rc_error(RC_ERR_OTHER, _("expression too long (>%d characters)"),
 		       MAX_EXPRESSION_LEN );
 		return;
 	}
@@ -1231,26 +1261,34 @@ void rCalc_engine_Evaluate( char *input )
 		if( strchr( rhs,'=' ) )
 		{
 			/* Engine | too many '=' in expression */
-			rc_error( _("too many equalities.") );
+			rc_error(RC_ERR_OTHER, _("too many equalities.") );
 			return;
 		}
 
 		name 	   = lhs;
 		expression = rhs;
 	}
-
+  
 	/*printf("name=%s and expression=%s\n",name,expression);*/
 
 	/* Evaluate the expression and store the result.
 	*/
+	
+	/*g_printf("expression ? : %s\n",expression);*/
+	init_expression = g_strdup(expression);
+
+  rc_export_value( name, init_expression, 0 );
+	
 	if( !breakInfix(&expression, exprInfix) ) return;
-	if( !infixToPostfix(&stack, exprInfix, exprPostfix) )  return;
+  if( !infixToPostfix(&stack, exprInfix, exprPostfix) )  return;
 	if( !evaluatePostfix( &result, &stack, exprPostfix ) ) return;
 	if( !setVariableValue( name, result ) ) return;
 
 	/* Print the result
 	*/
 	rc_printf("\t%s = %.16g\n", name, result );
+	rc_export_value( name, init_expression, result );
+	g_free(init_expression);
 }
 
 /*** end of engine/calculate.c ***********************************************/
