@@ -134,7 +134,7 @@ static int cmdExit( int argc, char **argv )
 	if( argc != 1 )
 	{
 		/* Engine | Commands | Prefix for format error messages */
-		rc_error( "%s %s", _("format:"), _(argv[0]) );
+		rc_error(RC_ERR_OTHER, "%s %s", _("format:"), _(argv[0]) );
 		return 0;
 	}
 	return 1;
@@ -146,7 +146,7 @@ static int cmdHelp( int argc, char **argv )
 	{
 		if( !strcasecmp( argv[0], "man" ) )
 		{
-			rc_error( "%s %s <%s>", _("format:"), argv[0], "topic" );
+			rc_error(RC_ERR_OTHER, "%s %s <%s>", _("format:"), argv[0], "topic" );
 			return 0;
 		}
 		rc_display_help( argv[1] );
@@ -156,7 +156,7 @@ static int cmdHelp( int argc, char **argv )
 		if(!( rc_display_help( argv[1] )))
 		{
 			/* Help | Error message for man and help */
-			rc_error(_("no help page available for `%s'."), argv[1]);
+			rc_error(RC_ERR_OTHER,_("no help page available for `%s'."), argv[1]);
 		}
 	}
 	return 0;
@@ -181,7 +181,7 @@ static int cmdList( int argc, char **argv )
 	*/
 	if( !strcasecmp( argv[0], "who" ) )
 	{
-		rc_error( "%s %s", _("format:"), argv[0] );
+		rc_error(RC_ERR_OTHER, "%s %s", _("format:"), argv[0] );
 		return 0;
 	}
 
@@ -276,7 +276,7 @@ static int cmdList( int argc, char **argv )
 			}
 		}
 		/* Engine | Commands | Error message for ls */
-		else rc_error( _("%s: unknown item."), argv[a] );
+		else rc_error(RC_ERR_OTHER, _("%s: unknown item."), argv[a] );
 	}
 	return 0;
 }
@@ -286,7 +286,6 @@ static int cmdRemove( int argc, char **argv )
 	VARIABLE *Variable = cfgCalc->variables;
 	FUNCTION *Function  = cfgCalc->functions;
 	int a,i,j;
-	char buff[2048];
 	G_CONST_RETURN gchar *home = g_get_home_dir();
 
 	g_assert(home);
@@ -297,7 +296,7 @@ static int cmdRemove( int argc, char **argv )
 	{
 		if( strcasecmp( argv[0], "clear" ) )
 		{
-			rc_error( "%s %s %s1 [%s2 ...]",
+			rc_error(RC_ERR_OTHER, "%s %s %s1 [%s2 ...]",
 			       _("format:"), argv[0],
 			       "variable", "variable" );
 			return 0;
@@ -362,7 +361,7 @@ static int cmdRemove( int argc, char **argv )
 			break;
 		}
 
-		if( !success ) rc_error( _("%s: undefined variable."), argv[a] );
+		if( !success ) rc_error(RC_ERR_UNDEFINED, _("%s: undefined variable."), argv[a] );
 	}
 
 	return 0;
@@ -388,7 +387,7 @@ static int cmdMode( int argc, char **argv )
 		}
 	}
 
-	rc_error( "%s %s [%s|%s]",
+	rc_error(RC_ERR_OTHER, "%s %s [%s|%s]",
 	       _("format:"), argv[0], "deg", "rad" );
 	return 0;
 }
@@ -399,7 +398,7 @@ static int cmdDegRad( int argc, char **argv )
 	
 	if( argc != 1 )
 	{
-		rc_error( "%s %s", _("format:"), argv[0] );
+		rc_error(RC_ERR_OTHER, "%s %s", _("format:"), argv[0] );
 		return 0;
 	}
 
@@ -496,7 +495,7 @@ int compile_function(char *name, char *expression, FUNCTION *function)
   }
 
   else {
-    rc_error( _("Could not compile function.\nCheck the syntax."));
+    rc_error(RC_ERR_OTHER, _("Could not compile function.\nCheck the syntax."));
     free(fullpath_o); free(fullpath_so); free(fullpath_c);
     free(command1); free(command2);
     return 0;
@@ -514,9 +513,9 @@ int compile_function(char *name, char *expression, FUNCTION *function)
 	error=(char *)g_module_error();
 	fputs (error, stderr);
 	if ( (suberror=strstr(error,"undefined")) != NULL)
-	  rc_error( _("%s\n"),suberror);
+	  rc_error(RC_ERR_OTHER, _("%s\n"),suberror);
 	else
-	  rc_error( _("%s\n"),error);
+	  rc_error(RC_ERR_OTHER, _("%s\n"),error);
 	
 	free(fullpath_o); free(fullpath_so); free(fullpath_c);
 	free(command1); free(command2);
@@ -530,7 +529,7 @@ int compile_function(char *name, char *expression, FUNCTION *function)
       {
     error =  (char *)g_module_error();
 	fprintf (stderr, "%s", error);
-	rc_error( _("error: %s\n"),error);
+	rc_error(RC_ERR_OTHER, _("error: %s\n"),error);
 	free(fullpath_o); free(fullpath_so); free(fullpath_c);
 	free(command1); free(command2);
 	return 0;
@@ -543,7 +542,7 @@ int compile_function(char *name, char *expression, FUNCTION *function)
   }
 
   else {
-    rc_error( _("Could not compile function.\nCheck the syntax."));
+    rc_error(RC_ERR_OTHER, _("Could not compile function.\nCheck the syntax."));
     free(fullpath_o); free(fullpath_so); free(fullpath_c);
     free(command1); free(command2);
     return 0;
@@ -583,13 +582,13 @@ static int cmdAddFunc ( int argc, char **argv )
 
   if( (fct_expr = strchr(buffer,'='))==NULL) 
 	{
-		rc_error( _("malformed expression.") );
+		rc_error(RC_ERR_OTHER, _("malformed expression.") );
 	}
 
   else {
     if ( (ptr=strchr(buffer,'('))==NULL  )
       {
-                rc_error( _("malformed expression.") );
+                rc_error(RC_ERR_OTHER, _("malformed expression.") );
       }
 
     else {
@@ -597,8 +596,17 @@ static int cmdAddFunc ( int argc, char **argv )
       namesize = strlen(buffer)-strlen(ptr)+1;
       fct_name = calloc(namesize,sizeof(char));
       strncpy(fct_name,buffer,namesize-1);
+      
+      rc_export_function( fct_name, fct_expr );
 
       if( !validVariableName(fct_name) ) return 0;
+      
+      if ( !verifyFunction(fct_expr) ) {
+        /* rc_error has already been called */
+        return 0;
+      }
+      
+      /* TODO: test here fct_expr validity */
 
       /* check if the function already exist */
       for( i=0; i<nFunctions; i++ )
@@ -606,7 +614,7 @@ static int cmdAddFunc ( int argc, char **argv )
 		if( strcasecmp( fct_name, Function[i].name ) ) continue;
 
 		if (i < NUM_DEFAULT_FUNCTIONS) {
-		  rc_error( _("This is already a default function\n") );
+		  rc_error(RC_ERR_OTHER, _("This is already a default function") );
 		  return 0;
 		}
 
@@ -619,7 +627,8 @@ static int cmdAddFunc ( int argc, char **argv )
 		  cfgCalc->nFunctions--;
 		  for( j=i; j<cfgCalc->nFunctions; j++)
 		    Function[j]=Function[j+1];
-		  rc_printf( _("Replacing existing function\n") );
+		  rc_debug( RC_DBG_ENGINE, _("Replacing existing function\n") );
+      rc_export_replace_state(TRUE);
 		}
 	}
 
@@ -637,12 +646,15 @@ static int cmdAddFunc ( int argc, char **argv )
 	strncpy(funct->name,fct_name,MAX_VARIABLE_NAME_LEN);
 	strncpy(funct->expression,fct_expr,MAX_FUNC_EXPR_LEN);
 	cfgCalc->nFunctions++;
+	rc_debug( RC_DBG_ENGINE, "%s: creating new function.", fct_name );
+  rc_export_replace_state(FALSE);
       /*}
       else {
 #ifndef USE_COMPILED_FUNCTIONS 
-	rc_error( _("malformed expression.") );
+	rc_error(RC_ERR_OTHER, _("malformed expression.") );
 #endif
       }*/
+    
     }
   }
 
@@ -694,7 +706,7 @@ static int executeCommand( COMMAND *c, char *command )
 	if(!( argv = calloc( argc, sizeof(char *) ) ))
 	{
 		/* Engine | Commands | Can't allocate any memory */
-		rc_error( _("not enough memory.") );
+		rc_error(RC_ERR_OTHER, _("not enough memory.") );
 		return 0;
 	}
 	for( line=command, i=0; i<argc; i++ )
@@ -745,6 +757,7 @@ static int doCommandOrExpression( char *command )
 			/* It is a command, so execute it...
 			*/
 			return_flag = executeCommand( c, command );
+			rc_export_command(command);
 		}
 		else
 		{
@@ -769,7 +782,7 @@ gboolean rc_engine_execute( RcEngine *engine, const gchar *command )
 	g_assert( engine );
 	g_assert( engine==cfgCalc );
 	g_assert( command );
-
+  
 	rc_debug( RC_DBG_ENGINE, "evaluating `%s'.", command );
 
 	copy = g_strdup( command );
