@@ -68,6 +68,23 @@ static gchar *angle_units[_RC_NUMBER_OF_ANGLE_UNITS]=
 
 #define MAX_PATH 2048
 
+void rc_engine_clean_state (RcEngine *engine)
+{
+  int new_nvar = engine->nVariables;
+  int i=0;
+  
+  g_assert( engine );
+  for( i=0; i < engine->nVariables; i++ )
+	{
+		if (!g_ascii_strncasecmp(engine->variables[i].name,"Ans",3)==0) {
+      new_nvar -= 1;
+     }    		
+	}
+  
+  engine->nVariables = new_nvar;
+  engine->nFunctions = NUM_DEFAULT_FUNCTIONS;
+}
+
 #ifdef RCALC_TEXT_ONLY
 FILE * text_open_config_file(char *flags)
 {
@@ -89,6 +106,7 @@ FILE * text_open_config_file(char *flags)
   /* NULL check has to be done later */
   return the_file;
 }
+#endif
 
 int text_config_get_int(FILE *cfg_file, char *cfg_string)
 {
@@ -112,6 +130,8 @@ int text_config_get_int(FILE *cfg_file, char *cfg_string)
 
     }
     while ( (!feof(cfg_file)) && (value < 0) );
+    
+   rewind (cfg_file);
 	 
 	 return value;
 }
@@ -138,7 +158,8 @@ double text_config_get_float(FILE *cfg_file, char *cfg_string)
     }
     while( (!feof(cfg_file)) && (value < 0));
 	 
-	 return value;
+	 rewind (cfg_file);
+   return value;
 }
 
 
@@ -164,10 +185,10 @@ gchar * text_config_get_string(FILE *cfg_file, char *cfg_string)
     }
     while( (!feof(cfg_file)) && (value == NULL));
 	 
+   rewind (cfg_file);
 	 return value;
 }
 
-#endif /* RCALC_TEXT_ONLY */
 
 /* returns function index that matches 'name', 
 or -1 if it does not exist.
@@ -314,7 +335,8 @@ RcEngine *rc_engine_new( void )
 	rCalc_engine_setDefaultFunctions(engine);
 	cfgCalc = engine;		/* TODO: remove */
 
-	return engine;
+	rc_init_impexp();
+  return engine;
 }
 
 /* Destructor.
@@ -349,7 +371,6 @@ void rc_engine_load_state_gnome( RcEngine *engine )
 	gint tmp_nfunctions;
 	int i;
 	gchar *angle_unit;
-	gchar *func_var_name;
 
 	g_assert( engine );
 	gnome_config_push_prefix( "/" PACKAGE "/engine/" );	
@@ -373,7 +394,7 @@ void rc_engine_load_state_gnome( RcEngine *engine )
 	}
 	g_free( angle_unit );
 
-        engine->func_var_name = gnome_config_get_string( "func_var_name=x" );
+  engine->func_var_name = gnome_config_get_string( "func_var_name=x" );
 
 	/* Read the variables.
 	*/
@@ -443,7 +464,6 @@ void rc_engine_load_state_gnome( RcEngine *engine )
 		gchar *name;
 		gchar *expr;
 		gchar key[32];
-		gint result;
 			
 		sprintf( key, "function[%d].name=%s", i, "<error>" );
 		name = gnome_config_get_string( key );
@@ -536,6 +556,11 @@ void rc_engine_load_state_no_gnome ( RcEngine *engine )
 	
 	config_file = text_open_config_file("r");
 	if (config_file == NULL) {
+	  config_version = 4;
+	  angle_unit = RC_ANGLE_DEGREES;
+	  engine->func_var_name="x";
+	  engine->nVariables=0;
+	  engine->nFunctions=0;
     return;
    }
 	
@@ -547,7 +572,7 @@ void rc_engine_load_state_no_gnome ( RcEngine *engine )
 	  g_assert( angle_unit<_RC_NUMBER_OF_ANGLE_UNITS );
 	  engine->angleMode = angle_unit;
 	  
-	  engine->func_var_name = text_config_get_string( "func_var_name=" );	  
+	  engine->func_var_name = text_config_get_string(config_file, "func_var_name=" );	  
 	  engine->nVariables = text_config_get_int(config_file, "num_variables=" );
 	  
 	  for( i=0; i<engine->nVariables; i++ )
